# :copyright: (c) 2021 by Pavlo Dmytrenko.
# :license: MIT, see LICENSE for more details.

"""
yaspin.base_spinner
~~~~~~~~~~~~~~~~~~~

Spinner class, used to construct other spinners.
"""
from dataclasses import dataclass


@dataclass
class Spinner:
    frames: str
    interval: int


default_spinner = Spinner("⠋⠙⠹⠸⠼⠴⠦⠧⠇⠏", 80)



"""
examples.basic_usage
~~~~~~~~~~~~~~~~~~~~

Common usage patterns for the yaspin spinner.
"""

import signal
import time

from yaspin import Spinner, yaspin
from yaspin.signal_handlers import fancy_handler
from yaspin.spinners import Spinners


def context_manager_default():
    with yaspin(text="Braille"):
        time.sleep(3)


def context_manager_line():
    line_spinner = Spinner("-\\|/", 150)
    with yaspin(line_spinner, "line"):
        time.sleep(3)


@yaspin(Spinner("⢄⢂⢁⡁⡈⡐⡠", 80), text="Dots")
def decorated_function():
    time.sleep(3)


def pre_setup_example():
    swirl = yaspin(
        spinner=Spinners.simpleDotsScrolling,
        text="swirl",
        color="red",
        side="right",
        sigmap={signal.SIGINT: fancy_handler},
    )

    with swirl as sp:
        time.sleep(2)

    with swirl as sp:
        sp.text = "new swirl"
        sp.reversal = True
        time.sleep(2)


def main():
    context_manager_default()
    context_manager_line()
    decorated_function()
    pre_setup_example()


if __name__ == "__main__":
    main()



"""
examples.dynamic_text
~~~~~~~~~~~~~~~~~~~~~

Evaluates text on every spinner frame.
"""

import time
from datetime import datetime

from yaspin import yaspin


class TimedText:
    def __init__(self, text):
        self.text = text
        self._start = datetime.now()

    def __str__(self):
        now = datetime.now()
        delta = now - self._start
        return f"{self.text} ({round(delta.total_seconds(), 1)}s)"


def main():
    with yaspin(text=TimedText("time passed:")):
        time.sleep(3)


if __name__ == "__main__":
    main()



"""
tests.test_timer
~~~~~~~~~~~~~~~~

Test timer feature.
"""

import re
import time

import pytest

from yaspin import yaspin


def test_no_timer():
    sp = yaspin(timer=False)
    sp._freeze("")

    assert re.search(r"\(\d+:\d{2}:\d{2}.\d{2}\)", sp._last_frame) is None


def test_timer_idle():
    sp = yaspin(timer=True)

    assert sp.elapsed_time == 0

    sp._freeze("")

    assert "(0:00:00.00)" in sp._last_frame


def test_timer_in_progress():
    sp = yaspin(timer=True)
    sp.start()

    t1 = sp.elapsed_time
    time.sleep(0.001)
    t2 = sp.elapsed_time

    sp.stop()

    assert t2 - t1 >= 0.001

    sp._freeze("")

    assert re.search(r"\(\d+:\d{2}:\d{2}.\d{2}\)", sp._last_frame) is not None


@pytest.mark.parametrize(
    "interval, expected", [(0.994, "(0:00:00.99)"), (0.996, "(0:00:01.00)")]
)
def test_timer_rounding(interval, expected):
    sp = yaspin(timer=True)
    sp.start()
    sp.stop()

    sp._stop_time = sp._start_time + interval
    sp._freeze("")

    assert expected in sp._last_frame


def test_timer_finished():
    sp = yaspin(timer=True)
    sp.start()

    time.sleep(0.001)

    sp.stop()

    assert sp.elapsed_time >= 0.001

    t1 = sp.elapsed_time
    time.sleep(0.001)
    t2 = sp.elapsed_time

    assert t1 == t2

    sp._freeze("")

    assert re.search(r"\(\d+:\d{2}:\d{2}.\d{2}\)", sp._last_frame) is not None
